#! /usr/bin/python

import os
import sys
import subprocess
import shutil
import optparse


TEST_TEMPLATE = """\
import sys
try:
    %(test)s
except:
    sys.exit(1)
else:
    sys.exit(0)
"""

DEFAULT_FILENAMES = [
    'a/Foobar.java',
    'a/b/Foo.java',
    'a/__init__.py',
    'a/b/__init__.py',
    'a/b/Bar.py',
    ]


tests = []

def create(filename):
    (root, ext) = os.path.splitext(filename)
    name = root.replace('/', '.')
    package_name = name[:name.rindex('.')]
    dir_name = package_name.replace('.', os.sep)
    last_name = name[name.rindex('.') + 1:]
    if not os.path.isdir(dir_name):
        os.makedirs(dir_name)

    if ext == '.py':
        template = '# module %(last_name)s;\n'
    elif ext == '.java':
        template = (
            'package %(package_name)s;\n'
            '\n'
            'class %(last_name)s {}\n'
            )

    open(filename, 'w').write(template % locals())

    if last_name != '__init__':
        tests.append('from %(package_name)s import %(last_name)s' % locals())
        tests.append('import %(package_name)s.%(last_name)s' % locals())


def iter_java_paths():
    for (dirpath, dirnames, filenames) in os.walk('.'):
        for filename in filenames:
            if filename.endswith('.java'):
                yield os.path.join(dirpath, filename)


def compile():
    subprocess.check_call(['javac'] + list(iter_java_paths()))


def prepare(filenames):
    shutil.rmtree('a', ignore_errors=True)
    for filename in filenames:
        create(filename)
    compile()


def main(args):
    parser = optparse.OptionParser(
        description='Exercise Jython import problem',
        )

    parser.set_defaults(tests=True)
    parser.add_option(
        '--tests', dest='tests', action='store_true',
        help='prepare tree and also run tests (default)',
        )
    parser.add_option(
        '--no-tests', dest='tests', action='store_false',
        help='only prepare tree; do not run tests',
        )

    parser.add_option(
        '--separate', action='store_true',
        help='run each test separately',
        )

    parser.add_option(
        '--jython', action='store', default='jython',
        help='set path to Jython executable',
        )

    (options, filenames) = parser.parse_args(args)

    if not filenames:
        filenames = DEFAULT_FILENAMES

    # Run the preparation once to determine the tests needed:
    prepare(filenames)

    if options.tests:
        test_list = list(tests)

        error = False

        for test in test_list:
            if options.separate:
                prepare(filenames)
            returncode = subprocess.call([options.jython, '-c', TEST_TEMPLATE % dict(test=test)])
            if returncode:
                sys.stderr.write('FAILURE: %s\n' % (test,))
                error = True
            else:
                sys.stderr.write('success: %s\n' % (test,))

        sys.exit(error)


main(sys.argv[1:])

